package OPCUATankSystem "Package containing the model used in the example"
  model ExampleModel "Example using the OPC UA library"
    OPCUA.Blocks.SamplerReal.Write writeReal(samplePeriod = 0.2, nodeID = "51031", nodeNamespace = 1, uaRealType = OPCUA.Types.UAReal.Double, nodeIDType = OPCUA.Types.UANodeIdType.NumericId) annotation(Placement(visible = true, transformation(origin = {9.867, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    OPCUA.OPCUAServer opcServer(hostName = "opc.tcp://localhost:16664") annotation(Placement(visible = true, transformation(origin = {50, 29.682}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    OPCUA.Blocks.SamplerReal.Read readReal(samplePeriod = 0.2, startTime = 0.1, nodeID = "51031", nodeNamespace = 1, y_start = 0, nodeIDType = OPCUA.Types.UANodeIdType.NumericId) annotation(Placement(visible = true, transformation(origin = {10, -10.318}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    OPCUA.Examples.Utilities.SimpleTank tank1(h(start = 1)) annotation(Placement(visible = true, transformation(origin = {-40, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Step step(offset = 0.25, height = 0.65, startTime = 5) annotation(Placement(visible = true, transformation(origin = {30, -70.318}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Continuous.LimPID PI(k = 1, Ti = 0.005, yMax = 1, Ni = 0.1, initType = Modelica.Blocks.Types.Init.InitialOutput, controllerType = Modelica.Blocks.Types.SimpleController.PI, Td = 0.1) annotation(Placement(transformation(extent = {{-56, -20}, {-36, 0}}, rotation = 0, origin = {101, -0}), visible = true));
    OPCUA.Examples.Utilities.ControlledTank tank2(h(start = 1), maxFlow = 1) annotation(Placement(visible = true, transformation(origin = {75, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Pulse pulse(amplitude = 0.45, period = 1, offset = 0, startTime = 7, nperiod = 1) annotation(Placement(visible = true, transformation(origin = {-80, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    connect(step.y, tank2.qIn) annotation(Line(visible = true, origin = {56.75, -67.659}, points = {{-15.75, -2.659}, {4.25, -2.659}, {4.25, 2.659}, {7.25, 2.659}}, color = {0, 0, 127}));
    connect(readReal.y, PI.u_s) annotation(Line(visible = true, origin = {32, -15.318}, points = {{-11, 5}, {0, 5.159}, {0, 5.159}, {11, 5.318}}, color = {0, 0, 127}));
    connect(tank2.y, PI.u_m) annotation(Line(visible = true, origin = {58, -57.333}, points = {{6, -17.667}, {-3, -17.667}, {-3, 35.333}}, color = {0, 0, 127}));
    connect(PI.y, tank2.u) annotation(Line(visible = true, origin = {82.5, -40.159}, points = {{-16.5, 30.159}, {6.5, 30.159}, {6.5, -24.841}, {3.5, -24.841}}, color = {0, 0, 127}));
    connect(tank1.h, writeReal.u) annotation(Line(visible = true, origin = {-15.42, 60}, points = {{-13.58, 0}, {13.58, 0}}, color = {0, 0, 127}));
    connect(opcServer.serverConnector, readReal.serverID) annotation(Line(visible = true, origin = {30.183, 9.682}, points = {{9.817, 20}, {-5.183, 20}, {-5.183, -15}, {-10.183, -15}}, color = {0, 170, 0}));
    connect(writeReal.serverID, opcServer.serverConnector) annotation(Line(visible = true, origin = {21.877, 44.682}, points = {{-2.01, 20.318}, {3.123, 20.318}, {3.123, -15}, {18.123, -15}}, color = {0, 170, 0}));
    connect(pulse.y, tank1.qIn) annotation(Line(visible = true, origin = {-60.5, 60}, points = {{-8.5, 0}, {8.5, 0}}, color = {0, 0, 127}));
    annotation(preferredView = "info", experiment(StopTime = 20, __Wolfram_SynchronizeWithRealTime = false), __Wolfram(PlotSet(plots = {Plot(name = "readReal.y and tank1.h", subPlots = {SubPlot(curves = {Curve(x = time, y = readReal.y), Curve(x = time, y = tank1.h)})}), Plot(name = "Documentation plot", subPlots = {SubPlot(curves = {Curve(x = time, y = writeReal.writeSignal), Curve(x = time, y = tank1.h)}), SubPlot(curves = {Curve(x = time, y = readReal.y), Curve(x = time, y = tank2.h)})})})), uses(Modelica(version = "3.2.3")), Documentation(info = "<html>


    <p>
This example shows how different parts of a process can be connected using the OPCUA Modelica library. The two processes in this model are a set of two tanks. One of the tanks <a href=\"modelica://OPCUA.Examples.Utilities.SimpleTank\">tanks</a> has an inflow of liquid that is controlled by a real-valued input signal and an output that is proportional to the volume of the liquid in the tank. The other <a href=\"modelica://OPCUA.Examples.Utilities.ControlledTank\">tank</a> has an inflow and outflow that is controlled by inputs.
</p><p>
The level of the liquid in the first tank is written to an OPC UA server using the OPCUA Library. That value is read in another part of the model and used as the reference signal for a PI controller that controls the level in the second tank by varying the outflow.
</p>

    <p>
At the beginning of the simulation, the first tank is filled with liquid to a height of 1 meter, while the second tank starts empty. The outflow from the first tank is proportional to the liquid level, causing an exponentially decreasing outflow. At seven seconds into the simulation, a pulse is sent to the tank that will give it a temporary inflow of liquid.
</p><p>
<img src=\"modelica://OPCUATankSystem/DocumentationFiles/Tank1Level.png\" alt=\"Tank1Level\"/>
</p><p>
This level is read from the tank and then written to a node on the OPC UA server. This node value is then read using the OPC UA protocol and used as an input to control the height of the liquid in the second tank.
</p><p>
<img src=\"modelica://OPCUATankSystem/DocumentationFiles/Tank2Level.png\" alt=\"Tank2Level\"/>
</p><p>
The second tank also has an inflow that is determined by the step component. A small spike can be seen in the liquid level at 5 s in the second tank that is due to this step component.
</p><a id=\"headerTag_OPCUAComponents\">&nbsp;</a>
    <h4>OPC UA Components</h4>
    <p>
What is used to communicate the signals here are the OPC UA components <a href=\"modelica://OPCUA.Blocks.SamplerReal.Read\">readReal</a>, <a href=\"modelica://OPCUA.Blocks.SamplerReal.Write\">writeReal</a>, and <a href=\"modelica://OPCUA.OPCUAServer\">OPCUAServer</a>. They are using external C code and communicating with the third-party library called open62541 (open62541.org). The server specified in the parameter \"hostName\" has to be running for this example to work correctly. Find more information by reading the <a href=\"modelica://OPCUA.GettingStarted\">Getting Started</a> guide included in the OPCUA Library.
</p></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, lineColor = {75, 138, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, lineColor = {0, 0, 255}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})}), Diagram(coordinateSystem(extent = {{-115, -100}, {110, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Rectangle(visible = true, origin = {-60, 59.841}, fillColor = {217, 223, 240}, fillPattern = FillPattern.Solid, extent = {{-40, -20.159}, {40, 20.159}}), Polygon(visible = true, origin = {35, -43.545}, fillColor = {217, 223, 240}, fillPattern = FillPattern.Solid, points = {{0, 53.545}, {60, 53.545}, {60, -46.773}, {-60, -46.773}, {-60, -6.773}, {0, -6.773}})}));
  end ExampleModel;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0"), OPCUA(version = "2.0")), experiment(__Wolfram_SynchronizeWithRealTime = false), __Wolfram(openClass = OPCUATankSystem.ExampleModel), Documentation, Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Text(visible = true, origin = {0.0, 21.0}, lineColor = {255, 0, 0}, extent = {{-120.0, 73.0}, {120.0, 132.0}}, textString = "%name", fontName = "Arial"), Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end OPCUATankSystem;
